@@ -1,7 +1,7 @@
 from mlpug.pytorch.multi_processing import MultiProcessingMixin
 
-from mlpug.trainers.callbacks.basic import *
 from mlpug.trainers.callbacks.basic import LogProgress as LogProgressBase
+from mlpug.trainers.callbacks.basic import BatchSizeLogger as BatchSizeLoggerBase
 
 
 class LogProgress(MultiProcessingMixin, LogProgressBase):
@@ -11,3 +11,7 @@ class LogProgress(MultiProcessingMixin, LogProgressBase):
 
         if not self.is_primary:
             self._log.warning("LogProgress is intended to be used only by the primary worker")
+
+
+class BatchSizeLogger(MultiProcessingMixin, BatchSizeLoggerBase):
+    pass
