@@ -23,15 +23,18 @@ class CheckpointManager(Callback, metaclass=abc.ABCMeta):
                  batch_level=True,
                  metric_to_monitor="validation.window_average.perplexity",
                  metric_opt_mode='min',
-                 metric_monitor_period=200,  # batches or epochs
+                 metric_monitor_period=None,  # batches or epochs
                  metric_checkpoint_threshold=None,
+                 allow_checkpointing_at_start=False,
                  create_checkpoint_every=200,  # batches or epochs
                  archive_last_model_checkpoint_every=2000,  # batches or epochs
-                 force_monitoring_on_epoch=True,
+                 create_latest_model_checkpoint=True,
+                 create_training_checkpoint=False,
                  base_checkpoint_filename=time.strftime("%d-%m-%Y_%H-%M-%S"),
                  model_checkpoint_filename_ext="m-ckp",
                  training_checkpoint_filename_ext="t-ckp",
                  backup_before_override=True,
+                 warn_on_model_quality_not_available=None,
                  name="CheckpointManager",
                  **kwargs):
         """
@@ -43,14 +46,21 @@ class CheckpointManager(Callback, metaclass=abc.ABCMeta):
                             on the epoch level
         :param metric_to_monitor: key path to metric value in the log object, e.g. `validation.batch.perplexity`
         :param metric_opt_mode: 'max', 'min'
-        :param metric_monitor_period: The period between checks for model quality improvement.
+        :param metric_monitor_period: If give, this is the period between checks for model quality improvement.
                                       This is in number of batches if `batch_level = True`, else it is a
                                       number of epochs.
+
+                                      If not given, every time a value is available for the given metric_to_monitor
+                                      the metric will be checked for improvement.
+
         :param metric_checkpoint_threshold: when given, the model quality must be below (or above, depending on
                                             metric_opt_mode) this threshold before a new best model checkpoint can be
                                             saved
 
-        :param create_checkpoint_every: period before saving next training/model checkpoint
+        :param allow_checkpointing_at_start: Default False. When True: when a value for the metric_to_monitor is
+                                             available at global batch iteration 0, checkpoints will be stored.
+
+        :param create_checkpoint_every: period before saving latest training/model checkpoint
                                         (will be overridden after each period)
                                         A 0 value disables this feature.
 
@@ -59,6 +69,12 @@ class CheckpointManager(Callback, metaclass=abc.ABCMeta):
 
                                                     A 0 value disables this feature.
 
+        :param create_latest_model_checkpoint:   If False, no latest model checkpoint will be created periodically
+                                                 (also see `create_checkpoint_every`)
+
+        :param create_training_checkpoint: If False, no training checkpoint will be created periodically
+                                           (also see `create_checkpoint_every`)
+
         :param force_monitoring_on_epoch: When True, the given metric will also be monitored on every epoch
                                           in the case that monitoring level is batch level
         :param base_checkpoint_filename:
@@ -68,6 +84,13 @@ class CheckpointManager(Callback, metaclass=abc.ABCMeta):
                                        overridden by a new version. If backing up fails, no new version will be written
                                        to disk. This gives the user a chance to fix a problem, e.g. disk full, without
                                        interruption of the training process.
+
+        :param warn_on_model_quality_not_available: Default is True when metric_monitor_period given else False.
+                                                    If set to true and the metric_to_monitor is not available in
+                                                    the logs, a warning will be logged.
+
+                                                    This is useful for debugging purposes
+
         """
         super().__init__(name=name, **kwargs)
 
@@ -85,13 +108,20 @@ class CheckpointManager(Callback, metaclass=abc.ABCMeta):
 
         self._metric_monitor_period = metric_monitor_period
         self._metric_checkpoint_threshold = metric_checkpoint_threshold
+        self._allow_checkpointing_at_start = allow_checkpointing_at_start
         self._create_checkpoint_every = create_checkpoint_every
         self._archive_last_model_checkpoint_every = archive_last_model_checkpoint_every
 
-        self._force_monitoring_on_epoch = force_monitoring_on_epoch
+        self._do_create_latest_model_checkpoint = create_latest_model_checkpoint
+        self._do_create_training_checkpoint = create_training_checkpoint
 
         self._backup_before_override = backup_before_override
 
+        if warn_on_model_quality_not_available is None:
+            warn_on_model_quality_not_available = self._metric_monitor_period is not None
+
+        self._warn_on_model_quality_not_available = warn_on_model_quality_not_available
+
         self._best_model_quality = float('Inf') if self._metric_opt_mode == 'min' else -float('Inf')
         self._best_model_iter = None
 
@@ -134,18 +164,19 @@ class CheckpointManager(Callback, metaclass=abc.ABCMeta):
         return self._monitor('global_iter', logs)
 
     def on_epoch_completed(self, logs):
-        iter_name = 'epoch'
+        create_latest_checkpoint = True
+        if not self._batch_level:
+            iter_name = 'epoch'
+        else:
+            # Monitor on epoch for batch level because checkpointing might be driven by
+            # availability of metric_to_monitor value (which could also have been calculated specifically
+            # at epoch ends.
+            iter_name = 'global_iter'
+            # ... but when monitoring on batch_level latest checkpoint might already be created
+            create_latest_checkpoint = False
 
-        force_monitoring = False
-        if self._batch_level:
-            if self._force_monitoring_on_epoch:
-                self._log.debug("Epoch completed : checking if model improved and creating checkpoints ... ")
-                iter_name = 'global_iter'
-                force_monitoring = True
-            else:
-                return True
-
-        return self._monitor(iter_name, logs, force_monitoring=force_monitoring)
+        self._log.debug("Epoch complete, checking if model quality improved ...")
+        return self._monitor(iter_name, logs, create_latest_checkpoint=create_latest_checkpoint)
 
     def on_training_ended(self, stopped_early, stopped_on_error, interrupted, callback_calls_success):
         status = 'ended'
@@ -163,14 +194,15 @@ class CheckpointManager(Callback, metaclass=abc.ABCMeta):
         self._log.info(f"Training {status}")
         success = True
         if self._create_checkpoint_every > 0:
-            self._log.info(f"... storing latest model ...")
-            latest_model_fname = self._create_model_checkpoint()
-            success = (latest_model_fname is not None)
+            if self._do_create_latest_model_checkpoint:
+                self._log.info(f"... storing latest model ...")
+                latest_model_fname = self._create_model_checkpoint()
+                success = (latest_model_fname is not None)
 
-        if stopped_early or interrupted:
-            self._log.info(f"... storing training checkpoint ...")
-            training_checkpoint_fname = self._create_training_checkpoint()
-            success &= (training_checkpoint_fname is not None)
+            if self._do_create_training_checkpoint and (status == 'ended' or stopped_early or interrupted):
+                self._log.info(f"... storing training checkpoint ...")
+                training_checkpoint_fname = self._create_training_checkpoint()
+                success &= (training_checkpoint_fname is not None)
 
         if abs(self._best_model_quality) != float('Inf') and self._best_model_iter is not None:
             iter_name = "global iteration" if self._batch_level else "epoch"
@@ -208,16 +240,33 @@ class CheckpointManager(Callback, metaclass=abc.ABCMeta):
 
     def _describe_setup(self):
         self._log.info(f"Metric to monitor : {self._metric_to_monitor}")
-        self._log.info(f"Metric monitor period : {self._metric_monitor_period}")
+        if self._metric_monitor_period is not None:
+            self._log.info(f"Metric monitor period : {self._metric_monitor_period}")
+        else:
+            self._log.info(f"Metric monitor period : Metric checked whenever it is available")
+
+            if self._allow_checkpointing_at_start:
+                self._log.info(f"Will create checkpoints at first global iteration, if a metric value is available")
 
         time_scale = 'batches' if self._batch_level else 'epochs'
-        self._log.info(f"Create last training & model checkpoints every "
-                       f"{self._archive_last_model_checkpoint_every} {time_scale}")
-        self._log.info(f"Archive last model checkpoint every "
-                       f"{self._archive_last_model_checkpoint_every} {time_scale}")
+        if self._create_checkpoint_every > 0:
+            if self._do_create_latest_model_checkpoint:
+                self._log.info(f"Create latest model checkpoints every "
+                               f"{self._create_checkpoint_every} {time_scale}")
+
+                self._log.info(f"Archive latest model checkpoint every "
+                               f"{self._archive_last_model_checkpoint_every} {time_scale}")
+
+            if self._do_create_training_checkpoint:
+                self._log.info(f"Create latest training checkpoints every "
+                               f"{self._create_checkpoint_every} {time_scale}")
+        else:
+            self._log.info(f"No latest model checkpoints and/or training checkpoint will be created")
 
     def _get_model_quality(self, current_logs):
-        model_quality = get_value_at(self._metric_to_monitor, current_logs)
+        model_quality = get_value_at(self._metric_to_monitor,
+                                     current_logs,
+                                     warn_on_failure=self._warn_on_model_quality_not_available)
 
         if type(model_quality) is tuple:
             # use the first value as metric value, the other values are auxiliary results meant for other purposes
@@ -226,7 +275,47 @@ class CheckpointManager(Callback, metaclass=abc.ABCMeta):
         return model_quality
 
     # TODO : it would maybe be better to split the monitoring and the checkpointing in to two separated methods
-    def _monitor(self, iter_name, logs, force_monitoring=False):
+    def _monitor(self, iter_name, logs, create_latest_checkpoint=True):
+        monitor_success = True
+        data_saved = False
+
+        current = self._get_logs_base(logs)
+        training_iter = current[iter_name]
+        if not self._allow_checkpointing_at_start and iter_name == "global_iter" and training_iter == 0:
+            self._log.debug("Skipping checkpointing at first iteration")
+            return monitor_success
+
+        success, best_model_fname = self._monitor_for_best_checkpoint(iter_name, logs)
+        monitor_success &= success
+        data_saved |= best_model_fname is not None
+
+        if create_latest_checkpoint:
+            success, \
+                latest_model_fname, \
+                training_checkpoint_fname = self._monitor_for_latest_checkpoint(iter_name,
+                                                                                logs,
+                                                                                best_model_fname=best_model_fname)
+            monitor_success &= success
+            data_saved |= latest_model_fname is not None
+            data_saved |= training_checkpoint_fname is not None
+
+            # latest_model_fname can be None when the user doesn't want to create latest checkpoints
+            if success and latest_model_fname is not None:
+                success, \
+                    archived_model_fname = self._monitor_for_checkpoint_archiving(iter_name,
+                                                                                  logs,
+                                                                                  latest_model_fname=latest_model_fname)
+                monitor_success &= success
+                data_saved |= archived_model_fname is not None
+            elif not success:
+                self._log.error("Unable to create checkpoint for latest model, will not archive latest model")
+
+        if data_saved:
+            self._log.debug("Saving of data done.\n\n")
+
+        return monitor_success
+
+    def _monitor_for_best_checkpoint(self, iter_name, logs):
         current = self._get_logs_base(logs)
 
         training_iter = current[iter_name]
@@ -235,99 +324,128 @@ class CheckpointManager(Callback, metaclass=abc.ABCMeta):
 
         best_model_fname = None
         success = True
-        data_saved = False
-        if force_monitoring or \
-                ((self._metric_monitor_period > 0) and (training_iter % self._metric_monitor_period == 0)):
-            model_quality = self._get_model_quality(current)
 
-            model_quality_valid = model_quality is not None
-            model_quality_good_enough = True
-            if model_quality_valid and self._metric_checkpoint_threshold is not None:
-                if self._metric_opt_mode == 'min':
-                    model_quality_good_enough = model_quality <= self._metric_checkpoint_threshold
-                elif self._metric_opt_mode == 'max':
-                    model_quality_good_enough = model_quality >= self._metric_checkpoint_threshold
+        check_model_quality = self._metric_monitor_period is None or \
+            ((self._metric_monitor_period > 0) and (training_iter % self._metric_monitor_period == 0))
 
-            if not model_quality_good_enough:
-                self._log.debug("%s : %s : model quality not good enough : %s : %3e " % (pretty_iter_name,
-                                                                                         training_iter,
-                                                                                         self._metric_to_monitor,
-                                                                                         model_quality))
+        if not check_model_quality:
+            return success, best_model_fname
 
-            if model_quality_valid and model_quality_good_enough:
-                model_improved = ((self._metric_opt_mode == 'min') and (model_quality < self._best_model_quality)) or \
-                                 ((self._metric_opt_mode == 'max') and (model_quality > self._best_model_quality))
+        model_quality = self._get_model_quality(current)
 
-                if model_improved:
-                    self._log.debug("%s : %s : Model improved : %s : %3e " % (pretty_iter_name,
-                                                                              training_iter,
-                                                                              self._metric_to_monitor,
-                                                                              model_quality))
+        model_quality_available = model_quality is not None
+        model_quality_good_enough = True
+        if model_quality_available and self._metric_checkpoint_threshold is not None:
+            if self._metric_opt_mode == 'min':
+                model_quality_good_enough = model_quality <= self._metric_checkpoint_threshold
+            elif self._metric_opt_mode == 'max':
+                model_quality_good_enough = model_quality >= self._metric_checkpoint_threshold
 
-                    if (self._best_model_iter is not None) and \
-                            (iter_name == 'epoch') and \
-                            (training_iter < self._best_model_iter):
-                        self._log.warn(f"Inconsistency: according to the current training {pretty_iter_name} "
-                                       f"({training_iter}), current best model training iter ({self._best_model_iter}) "
-                                       f"is in the future. Was the right training checkpoint loaded?")
+        if not model_quality_good_enough:
+            self._log.debug("%s : %s : model quality not good enough : %s : %3e " % (pretty_iter_name,
+                                                                                     training_iter,
+                                                                                     self._metric_to_monitor,
+                                                                                     model_quality))
 
-                    best_model_fname = self._save_current_model_as_best()
-                    if best_model_fname:
-                        self._best_model_quality = model_quality
-                        self._best_model_iter = training_iter
+        if model_quality_available and model_quality_good_enough:
+            model_improved = ((self._metric_opt_mode == 'min') and (model_quality < self._best_model_quality)) or \
+                             ((self._metric_opt_mode == 'max') and (model_quality > self._best_model_quality))
 
-                        data_saved = True
-                    else:
-                        self._log.error("Unable to save improved model checkpoint")
-                        success = False
+            if model_improved:
+                sys.stdout.write("\n")
+                sys.stdout.flush()
 
-                self._update_logs(model_improved, logs, current)
-            elif not model_quality_valid:
-                self._log.error(f"No model quality available, unable to check if we need to save a checkpoint, "
-                                f"skipping ...")
-                success = False
+                self._log.info("%s : %s : Model improved : %s : %3e " % (pretty_iter_name,
+                                                                         training_iter,
+                                                                         self._metric_to_monitor,
+                                                                         model_quality))
 
-        if (self._create_checkpoint_every > 0) and (training_iter % self._create_checkpoint_every == 0):
-            # Just copy best model if available
-            latest_model_fname = self._create_model_checkpoint(file_to_copy=best_model_fname)
+                if (self._best_model_iter is not None) and (training_iter < self._best_model_iter):
+                    self._log.warn(f"Inconsistency: according to the current training {pretty_iter_name.lower()} "
+                                   f"({training_iter}), current best model training {pretty_iter_name.lower()} "
+                                   f"({self._best_model_iter}) is in the future. "
+                                   f"Was the right training checkpoint loaded?")
 
-            latest_model_saved = (latest_model_fname is not None)
-            success &= latest_model_saved
-
-            data_saved |= latest_model_saved
-
-            if (self._archive_last_model_checkpoint_every > 0) and \
-                    (training_iter % self._archive_last_model_checkpoint_every == 0):
-                if latest_model_saved:
-                    copy_success = self._copy(latest_model_fname, self.current_model_file_name(training_iter))
-
-                    success &= copy_success
-
-                    data_saved |= copy_success
+                best_model_fname = self._save_current_model_as_best()
+                if best_model_fname is not None:
+                    self._best_model_quality = model_quality
+                    self._best_model_iter = training_iter
                 else:
-                    self._log.error("Unable to create checkpoint for latest model, unable to archive latest model")
+                    self._log.error("Unable to save improved model checkpoint")
                     success = False
 
+            self._update_logs(model_improved, logs, current)
+
+        return success, best_model_fname
+
+    def _monitor_for_latest_checkpoint(self, iter_name, logs, best_model_fname=None):
+        current = self._get_logs_base(logs)
+        training_iter = current[iter_name]
+
+        success = True
+        save_latest_model = (self._create_checkpoint_every > 0) and (training_iter % self._create_checkpoint_every == 0)
+
+        if not save_latest_model:
+            return success, None, None
+
+        latest_model_fname = None
+        if self._do_create_latest_model_checkpoint:
+            # Just copy best model if available
+            latest_model_fname = self._create_model_checkpoint(file_to_copy=best_model_fname)
+            success &= latest_model_fname is not None
+
+        training_checkpoint_fname = None
+        if self._do_create_training_checkpoint:
+            # Also save latest training checkpoint
             training_checkpoint_fname = self._create_training_checkpoint()
             training_checkpoint_success = (training_checkpoint_fname is not None)
-            data_saved |= training_checkpoint_success
 
             success &= training_checkpoint_success
 
-        if data_saved:
-            self._log.debug("Saving of data done.\n\n")
+        return success, latest_model_fname, training_checkpoint_fname
 
-        return success
+    def _monitor_for_checkpoint_archiving(self, iter_name, logs, latest_model_fname):
+        if latest_model_fname is None:
+            self._log.error("Unable to archive latest model, no latest checkpoint provided")
+            return False, None
+
+        current = self._get_logs_base(logs)
+        training_iter = current[iter_name]
+
+        archive_last_model = (self._archive_last_model_checkpoint_every > 0) and \
+                             (training_iter > 0) and \
+                             (training_iter % self._archive_last_model_checkpoint_every == 0)
+
+        success = True
+        if not archive_last_model:
+            return success, None
+
+        archived_model_fname = None
+        sys.stdout.write("\n")
+        sys.stdout.flush()
+
+        model_fname = self.current_model_file_name(training_iter)
+        copy_success = self._copy(latest_model_fname, model_fname)
+        if copy_success:
+            archived_model_fname = model_fname
+
+        success &= copy_success
+
+        return success, archived_model_fname
 
     def _save_current_model_as_best(self):
         model_fn = self.best_model_file_name()
 
         if self._backup_before_override:
+            err_msg = "A problem occurred backing up the last best model checkpoint, " \
+                      "will not override override model checkpoint with new one. \n\n" \
+                      "*** Please ensure there is enough disk space to store the backups and checkpoints ***\n\n"
             try:
-                self._backup_checkpoint(model_fn)
+                if not self._backup_checkpoint(model_fn):
+                    self._log.error(err_msg)
+                    return None
             except Exception as e:
-                _.log_exception(self._log, f"A problem occurred backing up the last best model checkpoint, "
-                                           f"will not override override model checkpoint with new one", e)
+                _.log_exception(self._log, err_msg, e)
                 return None
 
         return self._create_model_checkpoint(model_fn=model_fn)
@@ -452,7 +570,7 @@ class CheckpointManager(Callback, metaclass=abc.ABCMeta):
 
     def _copy(self, source_fname, dest_fname):
         try:
-            self._log.debug("Copying model: [%s] ==> [%s]" % (source_fname, dest_fname))
+            self._log.debug("Copying model:\n[%s] ==> \n[%s]\n" % (source_fname, dest_fname))
             copyfile(source_fname, dest_fname)
         except Exception as e:
             _.log_exception(self._log, "Unable to copy [%s]" % source_fname, e)
@@ -480,6 +598,28 @@ class CheckpointManager(Callback, metaclass=abc.ABCMeta):
                            f"changing archive_last_model_checkpoint_every to "
                            f"[{self._archive_last_model_checkpoint_every}]")
 
+    def _get_callback_properties_for_hash(self):
+        """
+        This is used to create the unique callback hash.
+
+        Returns a dict with properties describing the setup of the callback.
+        It should at least contain the properties that influence the callback state.
+
+        Property values should only be simple types, such as int, float, boolean and strings.
+        Convert any object and function values (or similar) into a booleans (True = available, False = None)
+
+        :return: dict
+        """
+        props = super()._get_callback_properties_for_hash()
+        return {
+            **props,
+            "batch_level": self._batch_level,
+            "metric_to_monitor": self._metric_to_monitor,
+            "metric_opt_mode": self._metric_opt_mode,
+            "metric_monitor_period": self._metric_monitor_period,
+            "metric_checkpoint_threshold": self._metric_checkpoint_threshold,
+        }
+
     @staticmethod
     def pretty_iter_name(iter_name):
         if iter_name == "epoch":
